/* Sophos Anti-Virus for OS/2 - Intercheck
   IchkStrt 2.10  February 2003
   Copyright (c) 2001,03 Sophos Plc, www.sophos.com
   Start optional Intercheck programs if installation is not invalid. */

CALL RxFuncAdd "SysLoadFuncs", "RexxUtil", "SysLoadFuncs"
CALL SysLoadFuncs

PARSE ARG StatusFile
IF StatusFile \= "" THEN DO
    StatusValue = GetStatusValue(StatusFile)
    IF StatusValue = -1 | StatusValue // 4 = 1 THEN
        CALL DisplayInvalidError
    IF StatusValue \= 0 THEN
        EXIT 0
END

PARSE SOURCE . . CmdPath
CmdDir = FILESPEC("D", CmdPath) || FILESPEC("P", CmdPath)

"@ECHO OFF"
"SETLOCAL"

SettingsFile = CmdDir || "INTERCHK.CFG"
IF FileExists(SettingsFile) THEN
    CALL ProcessCFG SettingsFile

IF StartSetting("MONITOR") THEN
    CALL StartProgram "MONITOR", CmdDir, ""

startPopup = StartSetting("POPUP")
startNotifier = StartSetting("NOTIFIER")
IF startPopup | startNotifier THEN DO
    PopupOptions = ""
    IF \ startPopup THEN
        PopupOptions = PopupOptions "p"
    IF \ startNotifier THEN
        PopupOptions = PopupOptions "n"
    CALL StartProgram "NOTIFY", CmdDir, PopupOptions
END

"ENDLOCAL"

EXIT 0


StartProgram: PROCEDURE
    PARSE ARG program, dir, options
    path = dir || program || ".EXE"
    IF FileExists(path) THEN
        "START /N" path options
    RETURN


/* Return 1 if given element is selected for starting, 0 otherwise. */
StartSetting: PROCEDURE
    ARG element
    c = TRANSLATE(LEFT(VALUE("START_" || element, , "OS2ENVIRONMENT"), 1))
    RETURN (c \= "N" & c \= "0")


FileExists: PROCEDURE
    PARSE ARG Filename
    RETURN STREAM(Filename, "C", "QUERY EXISTS") \== ""


GetStatusValue: PROCEDURE
    PARSE ARG Filename
    Filename = SubstituteFilename(Filename)
    Line = LINEIN(Filename)
    CALL STREAM Filename, "C", "CLOSE"
    Value = -1
    IF DATATYPE(Line, "W") THEN
        Value = 0 + Line
    RETURN Value


SubstituteFilename: PROCEDURE
    PARSE ARG Filename
    RETURN TRANSLATE(Filename, " ", "_")


DisplayInvalidError: PROCEDURE
    Topic = "SAV installation is invalid"
    Path = GetInfoPath()
    IF FileExists(Path) THEN
        "VIEW" Path """" || Topic || """"
    ELSE DO
        SAY "InterCheck for OS/2 cannot be started."
        SAY ""
        SAY "This can be because:"
        SAY ""
        SAY "- A manual install, update or remove operation failed, was interrupted"
        SAY "  or was cancelled."
        SAY ""
        SAY "- Information about the outcome of the last operation is corrupt."
        SAY ""
        SAY "Please perform the operation again or consult your system administrator."
        SAY ""
        "@PAUSE"
    END
    RETURN


GetInfoPath: PROCEDURE
    Path = ""
    Dir = GetSavDirectory()
    IF Dir \= "" THEN DO
        IF RIGHT(Dir, 1) \= "\" THEN
            Dir = Dir || "\"
        Path = Dir || "SETUP.INF"
    END
    RETURN Path


GetSavDirectory: PROCEDURE
    Dir = SysIni("USER", "SOPHOS_Anti-Virus", "Workstation")
    Dir = STRIP(Dir, "T", '0'X)
    IF Dir = "" | Dir = "ERROR:" THEN DO
        PARSE SOURCE . . CmdPath
        Dir = FILESPEC("D", CmdPath) || FILESPEC("P", CmdPath)
    END
    RETURN Dir


ProcessCFG:
    PROCEDURE
    ARG cfgFile

    e = STREAM( cfgFile, "C", "OPEN READ" )
    IF \ ABBREV( e, "READY" ) THEN
        RETURN 110                      /* ERROR_OPEN_FAILED */

    kAddress = "address"
    myAddress = TRANSLATE( STRIP( VALUE( "HOSTNAME",,"OS2ENVIRONMENT" ) ) )
    sectionList.0 = 0

    DO WHILE LINES( cfgFile )
        lyne = LINEIN( cfgFile )
        PARSE UPPER VAR lyne name "=" value .
        name = STRIP( name )
        SELECT
            WHEN name = "" THEN NOP
            WHEN LEFT( name, 1 ) == ';' THEN NOP
            WHEN ( LEFT( name, 1 ) == '[' ) & ( RIGHT( name, 1 ) == ']' ) THEN
                DO
                    section = sectionList.0 + 1
                    sectionList.0 = section
                    sectionList.section = name
                    tree.section.kAddress.0 = 0
                END
            OTHERWISE
                IF name == "ADDRESS" THEN
                DO
                    i = tree.section.kAddress.0 + 1
                    tree.section.kAddress.0 = i
                    tree.section.kAddress.i = value
                END
                ELSE
                    tree.section.name = value
        END
    END

    e = STREAM( cfgFile, "C", "CLOSE" )

    CALL ProcessSection "[INTERCHECKGLOBAL]", 1
    CALL ProcessSection "[INTERCHECKOS2GLOBAL]", 2
    CALL ProcessSection "[INTERCHECKWORKSTATION]", -1
    CALL ProcessSection "[INTERCHECKOS2WORKSTATION]", -2

    RETURN 0


ProcessSection:
    PROCEDURE EXPOSE kAddress myAddress sectionList. tree.
    ARG wantedName, type

    DO section = 1 TO sectionList.0
        IF wantedName == sectionList.section THEN
        DO
            IF type < 0 THEN
            DO i = 1 TO tree.section.kAddress.0
                IF myAddress == tree.section.kAddress.i THEN
                DO
                    type = -type
                    LEAVE i
                END
            END
            SELECT
                WHEN type = 1 THEN
                    DO
                        CALL ProcessOld section
                        LEAVE section
                    END
                WHEN type = 2 THEN
                    DO
                        CALL ProcessOS2 section
                        LEAVE section
                    END
                OTHERWISE NOP
            END
        END
    END
    RETURN


ProcessOld:
    PROCEDURE EXPOSE tree.
    ARG section

    CALL  SetEnv    "START_POPUP",      section,    "PopUpDisplay"
    RETURN


ProcessOS2:
    PROCEDURE EXPOSE tree.
    ARG section

    CALL  SetEnv    "START_POPUP",      section,    "VirusPopUp"
    CALL  SetEnv    "START_MONITOR",    section,    "StatusWindow"
    CALL  SetEnv    "START_NOTIFIER",   section,    "NetworkNotification"
    RETURN


SetEnv:
    PROCEDURE EXPOSE tree.
    PARSE UPPER ARG envname, section, cfgname

    IF SYMBOL( "tree.section.cfgname" ) == "VAR" THEN
    DO
        v = tree.section.cfgname
        SELECT
            WHEN v == "NO"  THEN x = "NO"
            WHEN v == "OFF" THEN x = "NO"
            WHEN v == "0"   THEN x = "NO"
            OTHERWISE x = "YES"
        END
        v = VALUE( envname, x, "OS2ENVIRONMENT" )
    END
    RETURN

