/* Sophos Anti-Virus for OS/2
   SETHNAME 1.00   September 2001
   Copyright (c) 2001 Sophos Plc, www.sophos.com
   Adjust CONFIG.SYS if necessary to include SET HOSTNAME. */

CALL RxFuncAdd "SysLoadFuncs", "RexxUtil", "SysLoadFuncs"
CALL SysLoadFuncs

ERROR_CODE = X2D("8000")
ERROR_COMPUTER = X2D("8004")
globals = "ERROR_CODE ERROR_COMPUTER"

IBMLANINIFile = SysSearchPath( "DPATH", "IBMLAN.INI" )
IF IBMLANINIFile == "" THEN
DO
    SAY "The IBM LAN Server network is not installed"
    EXIT 0
END

PARSE ARG ConfigLocation
IF ConfigLocation = "" THEN
DO
    ConfigFile = SysSearchPath( "DPATH", "OS2BOOT" )
    ConfigDir = FILESPEC("D", ConfigFile) || "\"
    ConfigFile = ConfigDir || "CONFIG.SYS"
END
ELSE
DO
    IF RIGHT(ConfigLocation, 1) == ":" THEN
        ConfigFile = ConfigLocation || "\CONFIG.SYS"
    ELSE
        ConfigFile = ConfigLocation
    ConfigDir = FILESPEC("D", ConfigFile) || FILESPEC("P", ConfigFile)
END
IF STREAM( ConfigFile, "C", "QUERY EXISTS" ) == "" THEN
DO
    SAY "Usage: ADJUST ConfigLocation"
    EXIT ERROR_CODE
END

IF AdjustmentNeeded( ConfigFile ) THEN
DO
    BackupFile = ConfigDir || "CONFIG.SAV"
    TempFile = ConfigDir || SysTempFileName( "SAV?????.TMP" )
    CALL AdjustConfig IBMLANINIFile, ConfigFile, BackupFile, TempFile
END

EXIT 0


AdjustmentNeeded: PROCEDURE EXPOSE (globals)
    PARSE ARG ConfigFile

    e = SysFileSearch( "HOSTNAME", ConfigFile, Lines. )
    IF e > 0 THEN
    DO
        SAY "Error examining CONFIG.SYS"
        EXIT ERROR_CODE
    END
    x = ""
    DO i = 1 TO Lines.0
        PARSE UPPER VAR Lines.i w1 w2 "=" w3 .
        IF (w1 == "SET") & (w2 = "HOSTNAME") THEN
            x = w3
    END
    RETURN ( x == "" )


GetActiveMachineName: PROCEDURE EXPOSE (globals)
    MachineName = ""
    "@NET START /NO >NUL 2>NUL"
    IF RC = 0 THEN
    DO
        q = RXQUEUE( "CREATE" )
        "@NET NAME | RXQUEUE" q "/FIFO 2>NUL"
        IF RC = 0 THEN
        DO
            q = RXQUEUE( "SET", q )
            e = QUEUED()
            DO WHILE e > 0
                PULL x
                e = QUEUED()
                IF ABBREV( x, "------------" ) THEN
                    LEAVE
            END
            IF e > 0 THEN
                PARSE UPPER PULL MachineName .
            q = RXQUEUE( "SET", q )
        END
        e = RXQUEUE( "DELETE", q )
    END
    RETURN MachineName


GetDefaultMachineName: PROCEDURE EXPOSE (globals)
    PARSE ARG IniFile

    e = STREAM( IniFile, "C", "OPEN READ" )
    IF \ ABBREV( e, "READY" ) THEN
    DO
        PARSE UPPER VAR e ":" err
        SAY "Error" err "opening" IniFile
        EXIT ERROR_CODE
    END

    ComputerName = ""
    found = 0
    DO WHILE LINES( IniFile )
        lyne = LINEIN( IniFile )
        PARSE UPPER VAR lyne name "=" value .
        name = STRIP( name )
        SELECT
            WHEN LEFT( name, 1 ) == ';' THEN NOP
            WHEN name == "" THEN NOP
            WHEN found THEN
                SELECT
                    WHEN ( LEFT( name, 1 ) == '[' ) & ,
                            ( RIGHT( name, 1 ) == ']' ) THEN LEAVE
                    WHEN name == "COMPUTERNAME" THEN
                        DO
                            ComputerName = value
                            LEAVE
                        END
                    OTHERWISE NOP
                END
            WHEN name == "[REQUESTER]" THEN
                found = 1
            OTHERWISE NOP
        END
    END

    e = STREAM( IniFile, "C", "CLOSE" )
    RETURN ComputerName


AdjustConfig: PROCEDURE EXPOSE (globals)
    PARSE ARG IniFile, ConfigFile, BackupFile, TempFile

    ComputerName = GetDefaultMachineName( IniFile )
    IF ComputerName == "" THEN
        ComputerName = GetActiveMachineName()
    IF ComputerName == "" THEN
    DO
        SAY "Error: cannot determine the network name of this machine"
        EXIT ERROR_COMPUTER
    END

    CALL CheckCopyFile ConfigFile, BackupFile
    "@ECHO SET HOSTNAME=" || ComputerName || ">" || TempFile
    IF RC > 0 THEN
    DO
        SAY "Error" RC "writing" TempFile
        EXIT ERROR_CODE
    END
    CALL CheckCopyFile TempFile '/B +' BackupFile '/B', ConfigFile
    CALL SysFileDelete TempFile
    RETURN


CheckCopyFile: PROCEDURE EXPOSE (globals)
    PARSE ARG src, dest
    "@COPY" src dest ">NUL"
    IF RC > 0 THEN
    DO
        SAY "Error" RC "copying" src "to" dest
        EXIT ERROR_CODE
    END
    RETURN
